// Controller for "view" related edit menu items.
// XXX - should be moved to view.js.
// XXX - note - we could probably get _significant_ performance
// gains by ensuring that this controller is _only_ in place when
// a view is current.  When we can guarntee this, all "isCommandEnabled()"
// calls, which currently call viewManager.getCurrentView(), can be replaced with
// "return true;".  This may already be true, but not going to
// attempt this optimization for release.


function editor_editorController() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
editor_editorController.prototype = new Controller();
editor_editorController.prototype.constructor = editor_editorController;

editor_editorController.prototype.destructor = function() {
}

// boolean prefs are marked "not supported" if there is no view
// This is to handle that "isCommandEnabled" is actually doing
// the state of the checkbox.
editor_editorController.prototype._is_bool_pref_command_supported = function(pref_name) {
    return gViewMgr.currentView != null;
}

// This is probably bogus.  Instead of asking scintilla its current state
// we should just ask the preference service for the state.
// See _is_bool_real_pref_command_enabled
editor_editorController.prototype._is_bool_pref_command_enabled = function(sci_prop_name) {
    var v = gViewMgr.currentView;
    if (v) {
        var exist = v.scintilla.scimoz[sci_prop_name];
        return exist;
    }
    return false;
}

editor_editorController.prototype._is_bool_real_pref_command_enabled = function(pref_name) {
    var cv = gViewMgr.currentView;
    if (cv) {
        return cv.prefs.getBooleanPref(pref_name)
    }
    return false;
}

editor_editorController.prototype._do_bool_pref_command = function(pref_name, existing_val) {
    gViewMgr.currentView.prefs.setBooleanPref(pref_name, !existing_val);
    // DONT update SciMoz here - let the preference notify mechanism
    // trigger the normal preference handling code.
}


//---- find stuff

editor_editorController.prototype.is_cmd_findNextSelected_enabled = function() {
    return gViewMgr.currentView ? true : false;
}

editor_editorController.prototype.do_cmd_findNextSelected = function() {
    if (findSvc == null) {
        findSvc = Components.classes["@activestate.com/koFindService;1"].
                  getService(Components.interfaces.koIFindService);
    }
    var v = gViewMgr.currentView;
    var scimoz = v.scintilla.scimoz
    if (!scimoz) return;
    var pattern = scimoz.selText;
    if (pattern == '') {
        scimoz.wordLeft();
        var curChar = scimoz.getWCharAt(scimoz.currentPos);
        while (curChar.search(/\W/) == -1) {
            pattern += curChar;
            scimoz.charRight();
            curChar = scimoz.getWCharAt(scimoz.currentPos);
        }
    }

    var context = Components.classes["@activestate.com/koFindContext;1"]
        .createInstance(Components.interfaces.koIFindContext);
    context.type = findSvc.options.preferredContextType;
    Find_FindNext(window, context, pattern);
}

editor_editorController.prototype.is_cmd_bookmarkToggle_enabled = function() {
    return gViewMgr.currentView != null;
}
editor_editorController.prototype.do_cmd_bookmarkToggle = function() {
    var v = gViewMgr.currentView;
    var line_no = v.scintilla.scimoz.lineFromPosition(v.scintilla.scimoz.selectionStart);
    var markerState = v.scintilla.scimoz.markerGet(line_no);
    if (markerState & (1 << MARKNUM_BOOKMARK)) {
        v.scintilla.scimoz.markerDelete(line_no, MARKNUM_BOOKMARK);
    } else {
        v.scintilla.scimoz.markerAdd(line_no, MARKNUM_BOOKMARK);
    }
}

editor_editorController.prototype.is_cmd_bookmarkRemoveAll_enabled = function() {
    return gViewMgr.currentView != null;
}
editor_editorController.prototype.do_cmd_bookmarkRemoveAll = function() {
    var v = gViewMgr.currentView;
    if (v) v.scintilla.scimoz.markerDeleteAll(MARKNUM_BOOKMARK);
}

editor_editorController.prototype.is_cmd_bookmarkGotoNext_enabled = function() {
    return gViewMgr.currentView != null;
}
editor_editorController.prototype.do_cmd_bookmarkGotoNext = function() {
    var v = gViewMgr.currentView;
    var thisLine = v.scintilla.scimoz.lineFromPosition(v.scintilla.scimoz.selectionStart);
    marker_mask = 1 << MARKNUM_BOOKMARK;
    var nextLine = v.scintilla.scimoz.markerNext(thisLine+1, marker_mask);
    if (nextLine < 0) {
        // try for search from top of file.
        nextLine = v.scintilla.scimoz.markerNext(0, marker_mask);
    }
    if (nextLine < 0 || nextLine == thisLine) {
        StatusBar_AddMessage("No next bookmark can be found.", "bookmark",
                             3000, true);
    } else {
        v.scintilla.scimoz.ensureVisibleEnforcePolicy(nextLine);
        v.scintilla.scimoz.gotoLine(nextLine);
    }
}


editor_editorController.prototype.is_cmd_bookmarkGotoPrevious_enabled = function() {
    return gViewMgr.currentView != null;
}
editor_editorController.prototype.do_cmd_bookmarkGotoPrevious = function() {
    var v = gViewMgr.currentView;
    var thisLine = v.scintilla.scimoz.lineFromPosition(v.scintilla.scimoz.selectionStart);
    marker_mask = 1 << MARKNUM_BOOKMARK;
    var prevLine = v.scintilla.scimoz.markerPrevious(thisLine-1, marker_mask);
    if (prevLine < 0) {
        // try for search from bottom of file.
        prevLine = v.scintilla.scimoz.markerPrevious(v.scintilla.scimoz.lineCount-1, marker_mask);
    }
    if (prevLine < 0 || prevLine == thisLine) {
        StatusBar_AddMessage("No previous bookmark can be found.", "bookmark",
                             3000, true);
    } else {
        v.scintilla.scimoz.ensureVisibleEnforcePolicy(prevLine);
        v.scintilla.scimoz.gotoLine(prevLine);
    }
}

editor_editorController.prototype.is_cmd_startMacroMode_enabled = function() {
    return gMacroMgr.mode != 'recording';
}

editor_editorController.prototype.do_cmd_startMacroMode= function() {
    gMacroMgr.startRecording();
}

editor_editorController.prototype.is_cmd_pauseMacroMode_enabled = function() {
    return gMacroMgr.mode == 'recording';
}

editor_editorController.prototype.do_cmd_pauseMacroMode= function() {
    gMacroMgr.pauseRecording();
}

editor_editorController.prototype.is_cmd_stopMacroMode_enabled = function() {
    return gMacroMgr.mode != 'stopped';
}

editor_editorController.prototype.do_cmd_stopMacroMode= function() {
    gMacroMgr.stopRecording();
}

editor_editorController.prototype.is_cmd_executeLastMacro_enabled = function() {
    return gMacroMgr.mode != 'recording' && gMacroMgr._currentMacro != null;
}

editor_editorController.prototype.do_cmd_executeLastMacro = function() {
    gMacroMgr.executeLastMacro()
}

editor_editorController.prototype.is_cmd_saveMacroToToolbox_enabled = function() {
    return gMacroMgr.mode != 'recording' && gMacroMgr._currentMacro != null;
}

editor_editorController.prototype.do_cmd_saveMacroToToolbox = function() {
    gMacroMgr.saveToToolbox();
}

editor_editorController.prototype.is_cmd_viewAsGuessedLanguage_enabled = function() {
    return (gViewMgr.currentView != null &&
      gViewMgr.currentView.getAttribute('type') == 'editor');
}

editor_editorController.prototype.do_cmd_viewAsGuessedLanguage = function() {
    try {
        gViewMgr.currentView.document.language = '';
        gViewMgr.currentView.document.language = gViewMgr.currentView.document.language;
    } catch (e) {
        log.exception(e);
    }
}

editor_editorController.prototype.is_cmd_browserPreview_enabled = function() {
    var view = gViewMgr.currentView;
    return (view != null &&
            view.getAttribute('type') == 'editor' &&
            view.document && view.document.file && view.document.file.isLocal);
}

editor_editorController.prototype.do_cmd_browserPreview = function() {
    var view = gViewMgr.currentView;
    var canPreview = (view != null &&
            view.getAttribute('type') == 'editor' &&
            view.document && view.document.file && view.document.file.isLocal);
    if (canPreview)
        gViewMgr.currentView.viewPreview();
}

editor_editorController.prototype.is_cmd_toolsBuildPerlAppCommand_enabled = function() {
    view = gViewMgr.currentView;
    if (view == null ||
        view.document.language != "Perl" ||
        view.document.isUntitled ||
        ! view.document.file.isLocal) {
        return false;
    }
    return true;
}

editor_editorController.prototype.do_cmd_toolsBuildPerlAppCommand = function() {
    try {
        var sysUtilsSvc = Components.classes["@activestate.com/koSysUtils;1"].
                          getService(Components.interfaces.koISysUtils);
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                      getService(Components.interfaces.koIInfoService);
        var interp = gPrefs.getStringPref("perlDefaultInterpreter");
        if (interp == '') {
            interp = 'perl';
            interp = sysUtilsSvc.Which(interp);
            if (!interp) {
                alert("This feature requires that you have " +
                      "Perl on your PATH or that the Perl default " +
                      "interpreter preference be correctly " +
                      "set, and that a licensed copy of Perl " +
                      "Dev Kit 3.0 or later be installed in " +
                      "the same directory.");
            }
        }
        var perlInfoEx = Components.classes["@activestate.com/koPerlInfoEx;1"].createInstance(Components.interfaces.koIPerlInfoEx);
        perlInfoEx.installationPath = perlInfoEx.getInstallationPathFromBinary(interp);
        var sysDataSvc = Components.classes["@activestate.com/koSystemDataService;1"]
                         .getService(Components.interfaces.koISystemDataService);
        
        var perlBinDir = os.path.dirname(interp)
        sysDataSvc.setString("pdk.perlbinpath", perlBinDir);

        if (! sysDataSvc.getString('pdk.perlapp')) {
            alert("Komodo could not find an installation of Perl Dev Kit in: '" + perlBinDir + "'.");
            return;
        }
        if (! sysDataSvc.getBoolean("pdk.haveLicense")) {
            alert("This feature requires that you have " +
                  "a license for the Perl Dev Kit installed.");
            return;
        }
        var version = sysDataSvc.getString("pdk.version");
        if (! version) {
            alert("Komodo could not determine the version number for Perl Dev Kit.");
            return;
        }
        if (parseFloat(version) < 3) {
            alert("Komodo requires Perl Dev Kit version 3 or later.");
            return;
        }
    } catch (e) {
        log.error(e);
        return;
    }
    launch_perlappWindow();
}

window.controllers.appendController(new editor_editorController());
