/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

// Open the given URI and return the view created.
//
//  "uri" is the URI to open, it may be a URI or a local path.
//  "viewType" (optional, default "editor") can be used to specify the
//      view type in which to open the URI.
//  "skipRecentOpenFeature" (optional, default "false") can be used when
//      the URI to open is a project file to specify that the feature to
//      open files in that project should not be offered.
//
// If the URI is not successfully opened or if the URL opened is a Komodo
// project file null is returned.
function open_openURI(uri, viewType /* ="editor" */,
                      skipRecentOpenFeature /* =false */)
{
    // URI can be a local path or a URI
    uri = uriparse_pathToURI(uri);
    if (typeof(viewType)=='undefined' || !viewType)
        viewType = 'editor'; 
    if (uri.match(/\.kpf$/i)) {
        project_openProjectFromURL(uri, skipRecentOpenFeature);
    } else if (uri.match(/\.kpz$/i)) {
        package_ImportPackage(toolboxMgr, null, uriparse_URIToLocalPath(uri));
// #if PRODUCT_TYPE == "professional"
    } else if (uri.match(/\.ui$/i)) {
        try {
            var guiBuilder = Components.classes["@activestate.com/koGuiBuilderService;1"].
                        getService(Components.interfaces.koIGuiBuilderService);
            guiBuilder.Open(uri, '');
        } catch (e) {
            log.error(e);
        }
// #endif
    } else {
        return gViewMgr.doFileOpen(uri,viewType);
    }
    return null;
}


// Open the given path in Komodo.
// 
//  "displayPath" identifies the path to open. Display path may be the display
//      path of an already open (and possibly untitled) document.
//  "viewType" (optional) is the type of view to create for the openned path.
//      It is ignored if the displayPath indicates an already open view.
//      Otherwise it defaults to "editor" if not specified.
//
function open_openDisplayPath(displayPath, viewType /* ="editor" */)
{
    if (typeof(viewType) == "undefined" || !viewType) viewType = "editor"; 

    var views = gViewMgr.topView.getViews(true);
    for (var i = 0; i < views.length; ++i) {
        if (views[i].document && views[i].document.displayPath == displayPath) {
            views[i].makeCurrent();
            return;
        }
    }

    // Fallback to open URI.
    open_openURI(displayPath);
}


// Open Komodo's Start Page
//
function open_openStartPage()
{
    gViewMgr.doFileOpen("chrome://jaguar/content/startpage/startpage.xml#view-startpage",
                        "startpage");
}


function open_openMultipleURIs(urls, viewType)
{
    var i,j;
    if (urls.length) {
        var viewStateMRU = gPrefSvc.getPrefs("viewStateMRU");
        var projectFiles = [];
        var projectViewState,file_url;
        for (i=0; i < urls.length; i++) {
            if (viewStateMRU.hasPref(urls[i])) {
                projectViewState = viewStateMRU.getPref(urls[i]);
                if (projectViewState.hasPref('opened_files')) {
                    opened_files = projectViewState.getPref('opened_files');
                    if (opened_files.length > 0) {
                        for (j=0; j < opened_files.length; j++) {
                            file_url = opened_files.getStringPref(j);
                            projectFiles.push(file_url);
                        }
                    }
                }
            }
        }
        
        var action;
        if (projectFiles.length > 0) {
            action = dialog_yesNoCancel(
                "One or more projects you are opening had files open "+
                "when you closed them.  Would you like to reopen "+
                "these files?",
                "Yes", null, null, // default response, text, title
                "open_recent_files_on_project_open");
            if (action == "Cancel") {
                return;
            }
            if (action == "Yes") {
                urls = urls.concat(projectFiles);
            }
        }
        
        if (urls.length > 1) {
            gViewMgr.batchMode = true;
        }
        for (i=0; i < urls.length; i++) {
            if (i == urls.length-1) {
                gViewMgr.batchMode = false;
            }
            open_openURI(urls[i], viewType, true);
        }
    }
}


// Open and read and return a pixmap.
//
// Note: The pixmap is cached by URI.
// This is used to load pixmaps for scintilla markers.
var _gPixmaps = {};
function open_readPixmap(uri) {
    if (typeof(_gPixmaps[uri]) != 'undefined') {
        return _gPixmaps[uri];
    }
    var file = Components.classes["@activestate.com/koFileEx;1"] 
            .createInstance(Components.interfaces.koIFileEx)
    file.URI = uri;
    file.open('rb');
    _gPixmaps[uri] = file.readfile();
    file.close(); 
    return _gPixmaps[uri];
}

