/* Copyright (c) 2004 ActiveState Corp, a division of Sophos Plc.
   See the file LICENSE.txt for licensing information. */


//---- globals

var log = getLoggingMgr().getLogger("pref-codeintel");
//log.setLevel(LOG_INFO);

var _elementIDs = [
    "codeintel_enabled",
    "codeintel_completion_triggering_enabled",
    "codeintel_rescan_while_typing_enabled",

    //"codeintel_scanning_imports_enabled",

];

var gWidgets = new Object();



//---- internal support functions

function _setElementEnabledState(elt, enabled) {
    if (enabled) {
        if (elt.hasAttribute('disabled')) {
            elt.removeAttribute('disabled');
        }
    } else {
        elt.setAttribute('disabled', true);
    }
}

function _englishJoinList(list, conjunction /* ="and" */) {
    if (typeof(conjunction) == "undefined" || conjunction == null)
        conjunction = "and";
    var s = "";
    if (list.length > 0) {
        s += list[0];
        for (i = 1; i < list.length-1; ++i) {
            s += ", "+list[i];
        }
        if (list.length > 1) {
            s += " "+conjunction+" "+list[list.length-1];
        }
    }
    return s;
}



//---- functions for XUL

function PrefCodeIntel_OnLoad()
{
    log.info("PrefCodeIntel_OnLoad");
    try {
        gWidgets.enabledCheckbox = document.getElementById("codeintel_enabled");
        gWidgets.triggeringEnabledCheckbox = document.getElementById("codeintel_completion_triggering_enabled");
        gWidgets.rescanningEnabledCheckbox = document.getElementById("codeintel_rescan_while_typing_enabled");

        gWidgets.notPreparedDesc = document.getElementById("notPreparedForLanguagesWarning");
        gWidgets.buildCIDBButton = document.getElementById("buildCIDBButton");



        //gWidgets.scanningImportsEnabledCheckbox = document.getElementById("codeintel_scanning_imports_enabled");


        var tag = "chrome://jaguar/content/pref/pref-codeintel.xul";
        parent.hPrefWindow.onpageload(tag);


        PrefCodeIntel_UpdatePrepUI();

        PrefCodeIntel_UpdateCodeIntelUI();
    } catch(ex) {
        log.exception(ex);
    }
}



function PrefCodeIntel_UpdatePrepUI()
{
    try {
        // Determine which, if any, languages have not been prepared for
        // completion.
        var allLangs = parent.opener.gPrefs.getStringPref("codeintel_completion_languages");
        allLangs = allLangs.split(/\s+/);
        var preppedLangs = parent.opener.gPrefs.getStringPref("codeintel_languages_prepped_for_completion");
        preppedLangs = preppedLangs.split(/\s+/);
        var unpreppedLangs = [];
        for (var a = 0; a < allLangs.length; a++) {
            var foundIt = false;
            for (var p = 0; p < preppedLangs.length; p++) {
                if (allLangs[a] == preppedLangs[p]) {
                    foundIt = true;
                    break;
                }
            }
            if (!foundIt) {
                unpreppedLangs.push(allLangs[a]);
            }
        }

        var desc = document.getElementById("notPreparedForLanguagesWarning");
        if (unpreppedLangs.length == 0) {
            gWidgets.notPreparedDesc.setAttribute("collapsed", true);
        } else {
            if (gWidgets.notPreparedDesc.hasAttribute("collapsed"))
                gWidgets.notPreparedDesc.removeAttribute("collapsed");
            var note = "Note: Komodo's Code Intelligence database has not "+
                       "been yet been built for "+
                       _englishJoinList(unpreppedLangs, "and")+
                       ", or is out of date. This can affect the accuracy "+
                       "and usefulness of AutoComplete and CallTips. "+
                       "Click 'Scan language installations' to "+
                       "prepare Komodo's Code Intelligence system now.";
            if (desc.childNodes.length > 0) {
                desc.removeChild(desc.childNodes[0]);
            }
            desc.appendChild(document.createTextNode(note));
        }
    } catch(ex) {
        log.exception(ex);
    }
}


function PrefCodeIntel_UpdateCodeIntelUI()
{
    try {
        var enabled = gWidgets.enabledCheckbox.checked;
        _setElementEnabledState(gWidgets.rescanningEnabledCheckbox, enabled);

        _setElementEnabledState(gWidgets.buildCIDBButton, enabled);



        _setElementEnabledState(gWidgets.triggeringEnabledCheckbox, enabled);
        //_setElementEnabledState(gWidgets.scanningImportsEnabledCheckbox, enabled);

    } catch(ex) {
        log.exception(ex);
    }
}


function PrefCodeIntel_BuildCIDB(askForDirs)
{
    try {
        var args = new Object();
        args.askForDirs = askForDirs;
        args.activePrefs = parent.hPrefWindow.prefset;
        parent.opener.openDialog("chrome://jaguar/content/codeintel/buildcidb.xul",
                                 "komodo_buildcidb",
                                 "chrome,dialog=no,close=yes,dependent,resizable",
                                 args);
    } catch(ex) {
        log.exception(ex);
    }
}

function PrefCodeIntel_ImportCIXFile()
{
    try {
        var path = filepicker_openFile(null, null,
                                       "Import CIX File",
                                       "XML",
                                       ["XML", "All"]);
        if (!path) return;
        
        var codeIntelSvc = Components.classes["@activestate.com/koCodeIntelService;1"]
                           .getService(Components.interfaces.koICodeIntelService);
        codeIntelSvc.batchUpdateRequest("cix", path, null, null);
        codeIntelSvc.batchUpdateStart(null);

        //XXX Should eventually have a "progress" dialog here if it is
        //    taking a long time.
        codeIntelSvc.batchUpdateWait();

        var countObj = new Object();
        var errors = codeIntelSvc.batchUpdateGetErrors(countObj, errors);
        if (errors && errors.length) {
            dialog_alert("Could not import CIX file into Code Intelligence Database.",
                         errors.join('\n'),
                         "CIX Import Error");
        }
    } catch(ex) {
        log.exception(ex);
    }
}


