from xpcom import components
sm = components.interfaces.ISciMoz

from factoryStyleSettings import \
     CommonFactoryStyles, LanguageFactoryStyles
from userStyleSettings import \
     CommonCustomStyles, LanguageCustomStyles

DefaultStyle = CommonFactoryStyles['default'].copy()
DefaultStyle.update(CommonCustomStyles.get('default', {}))
from styles import StateMap

ScimozStyleNo2CommonName = {}
ValidStyles = {}
AppliedStyles = {}
languages = StateMap.keys()

_gNeedUpdate = 1

def buildCompoundData():
    """ This takes the factory settings both for common styles and
    language-unique styles and combines them with the user-specified
    version for both sets of styles. """
    for language in languages:
        ValidStyles[language] = []
        for common_name, scimoz_names in StateMap[language].items():
            #print scimoz_names
            for scimoz_name in scimoz_names:
                if isinstance(scimoz_name, int):
                    scimoz_no = scimoz_name
                else:
                    scimoz_no = getattr(sm, scimoz_name)
                ScimozStyleNo2CommonName[scimoz_no] = common_name
                ValidStyles[language].append((scimoz_no,
                                                scimoz_name,
                                                common_name)) 
    
        langStyle = AppliedStyles[language] = CommonFactoryStyles.copy()
        langStyle.update(LanguageFactoryStyles.get(language, {}))
        langStyle.update(CommonCustomStyles)
        langStyle.update(LanguageCustomStyles.get(language, {}))
    _gNeedUpdate = 0

def SerializeUserData():
    f = open('userStyleSettings2.py', 'w')
    import pprint
    f.write('CommonCustomStyles = ')
    pprint.pprint(CommonCustomStyles)
    f.write('\n')
    f.write('LanguageCustomStyles= ')
    pprint.pprint(LanguageCustomStyles)
    f.write('\n')

def setStyleData(language,   # pass in None to set common style
                 style_name, # pass in one of the style names
                 aspect,     # font, fore, back, bold, italic, or eolfilled
                 value,      # pass in None to clear
                 ):
    if language is None:
        if value is None:
            del CommonCustomStyles[style_name][aspect]
        else:
            CommonCustomStyles[style_name][aspect] = value
    else:
        if value is None:
            del LanguageCustomStyles[language][style_name][aspect]
        else:
            LanguageCustomStyles[language][style_name][aspect] = value
    global _gNeedUpdate
    _gNeedUpdate = 1


def getStyleByNumber(language, styleNumber):
    name = ScimozStyleNo2CommonName[styleNumber]
    for (scimoz_no, scimoz_name, common_name) in ValidStyles[language]:
        if common_name == name:
            break
    else:
        #print "COULDN't FIND: ", name
        return Style({})
    # XXX -- doesn't deal with the font, I don't think.
    langStyle = AppliedStyles[language]
    langStyleData = langStyle[common_name]
    return Style(langStyle[common_name])


class Style:
    fore = 0
    back = 0xffffff
    font = 'courier'
    size = '10'
    italic = 0
    bold = 0
    def __init__(self, data):
        self.__dict__.update(data)
        if 'fore' in data:
            self.fore = '#' + hex(data['fore'])[2:]
        #print data

def applyStylesForLanguage(scimoz, language, encoding, useFixed):
    setters = {
        'fore': scimoz.styleSetFore,
        'back': scimoz.styleSetBack,
        'bold': scimoz.styleSetBold,
        'italic': scimoz.styleSetItalic,
        'size': scimoz.styleSetSize,
        'eolfilled': scimoz.styleSetEOLFilled,
    }
    if _gNeedUpdate:
        buildCompoundData()

    # We need to build up the default style for this language
    langDefaultStyle = DefaultStyle.copy()
    langd = AppliedStyles[language].get('default', {})
    langDefaultStyle.update(langd)

    # set default style
    for aspect, setter in setters.items():
        value = langDefaultStyle[aspect]
        setter(scimoz.STYLE_DEFAULT, value)
    if useFixed:
        font = 'font_' + encoding + '_fixed'
        defaultFont = 'font_default_fixed'
    else:
        font = 'font_' + encoding + '_proportional'
        defaultFont = 'font_default_proportional'
    
    scimoz.styleSetFont(scimoz.STYLE_DEFAULT,
                        langDefaultStyle.get(font,
                            langDefaultStyle[defaultFont]))

    scimoz.styleClearAll() # now all styles are the same
    
    langStyle = AppliedStyles[language]
    for (scimoz_no, scimoz_name, common_name) in ValidStyles[language]:
        if not common_name in langStyle:
            print "missing style data for style %s of language %s" %\
                  (common_name, language)
            continue

        # first deal with the font
        langStyleData = langStyle[common_name]
        stylefont = langStyleData.get(font, None)
        if stylefont and stylefont != langDefaultStyle.get(font,
                                                       'default_font'):
            scimoz.styleSetFont(scimoz_no, stylefont)

        for aspect, setter in setters.items():
            value = langStyle[common_name].get(aspect, None)
            if value and value != langDefaultStyle[aspect]:
                #print aspect, scimoz_name, value
                setter(scimoz_no, value)
                
if __name__ == '__main__':
    buildCompoundData()
    #import pprint
    #pprint.pprint(AppliedStyles['Perl'])
    #class FakeScimoz:
    #    def styleSetFont(self, styleNo, font):
    #        print "setting font of style %d to %s" % (styleNo, font)
    #    def styleSetSize(self, styleNo, color):
    #        print "setting size of style %d to %d" % (styleNo, color)
    #    def styleSetFore(self, styleNo, color):
    #        print "setting fore of style %d to %d" % (styleNo, color)
    #    def styleSetBack(self, styleNo, color):
    #        print "setting back of style %d to %d" % (styleNo, color)
    #    def styleSetBold(self, styleNo, color):
    #        print "setting bold of style %d to %d" % (styleNo, color)
    #    def styleSetItalic(self, styleNo, color):
    #        print "setting italic of style %d to %d" % (styleNo, color)
    #    def styleSetEOLFilled(self, styleNo, bool):
    #        print "setting EOLFilled of style %d to %s" % (styleNo, bool)
    #    def styleClearAll(self):
    #        print "Clearing all styles"
    #    STYLE_DEFAULT=32
    #fakescimoz = FakeScimoz()
    #applyStylesForLanguage(fakescimoz, 'Python', 'cp1252', 1)
    #applyStylesForLanguage(fakescimoz, 'Python', 'iso8859-15', 1)
    #applyStylesForLanguage(fakescimoz, 'Python', 'iso8859-15', 0)
    #applyStylesForLanguage(fakescimoz, 'Python', 'iso8859-2', 0)
    getStyleByNumber('Python', 12)
    #SerializeUserData()

 
